from bs4 import BeautifulStoneSoup
import io
import magic
import os
from zipfile import ZipFile
from trytond.exceptions import UserError

class XmlAttach():
    xml_attach_string = """
    <AttachedDocument xmlns="urn:oasis:names:specification:ubl:schema:xsd:AttachedDocument-2" xmlns:ds="http://www.w3.org/2000/09/xmldsig#"
    xmlns:cac="urn:oasis:names:specification:ubl:schema:xsd:CommonAggregateComponents-2"
    xmlns:cbc="urn:oasis:names:specification:ubl:schema:xsd:CommonBasicComponents-2"
    xmlns:ccts="urn:un:unece:uncefact:data:specification:CoreComponentTypeSchemaModule:2"
    xmlns:ext="urn:oasis:names:specification:ubl:schema:xsd:CommonExtensionComponents-2" xmlns:xades="http://uri.etsi.org/01903/v1.3.2#"
    xmlns:xades141="http://uri.etsi.org/01903/v1.4.1#">
    <cbc:UBLVersionID>UBL 2.1</cbc:UBLVersionID>
    <cbc:CustomizationID>Documentos adjuntos</cbc:CustomizationID>
    <cbc:ProfileID>DIAN 2.1</cbc:ProfileID>
    <cbc:ProfileExecutionID>2</cbc:ProfileExecutionID>
    <cbc:ID>1234</cbc:ID>
    <cbc:IssueDate>2019-04-05</cbc:IssueDate>
    <cbc:IssueTime>03:37:11-05:00</cbc:IssueTime>
    <cbc:DocumentType>Contenedor de Factura Electrónica</cbc:DocumentType>
    <cbc:ParentDocumentID>SETSG980000018</cbc:ParentDocumentID>
    <cac:SenderParty>
    <cac:PartyTaxScheme>
    <cbc:RegistrationName>Sociedad de Colombia SAS</cbc:RegistrationName>
    <cbc:CompanyID schemeAgencyID="195" schemeID="8" schemeName="31">700081234</cbc:CompanyID>
    <cbc:TaxLevelCode listName="05">O-06</cbc:TaxLevelCode>
    <cac:TaxScheme>
    <cbc:ID>01</cbc:ID>
    <cbc:Name>IVA</cbc:Name>
    </cac:TaxScheme>
    </cac:PartyTaxScheme>
    </cac:SenderParty>
    <cac:ReceiverParty>
    <cac:PartyTaxScheme>
    <cbc:RegistrationName>TEST RECEPTOR DOCUMENTO</cbc:RegistrationName>
    <cbc:CompanyID schemeAgencyID="195" schemeID="9" schemeName="31">900508908</cbc:CompanyID>
    <cbc:TaxLevelCode listName="05">O-06</cbc:TaxLevelCode>
    <cac:TaxScheme>
    <cbc:ID>01</cbc:ID>
    <cbc:Name>IVA</cbc:Name>
    </cac:TaxScheme>
    </cac:PartyTaxScheme>
    </cac:ReceiverParty>
    <cac:Attachment>
    <cac:ExternalReference>
    <cbc:MimeCode>text/xml</cbc:MimeCode>
    <cbc:EncodingCode>UTF-8</cbc:EncodingCode>
    <cbc:Description><![CDATA[Acá se coloca el DE en formato xml]]></cbc:Description>
    </cac:ExternalReference>
    </cac:Attachment>
    <cac:ParentDocumentLineReference>
    <cbc:LineID>1</cbc:LineID>
    <cac:DocumentReference>
    <cbc:ID>SETSG980000018</cbc:ID>
    <cbc:UUID schemeName="CUFE-SHA384">16f8e126477fbddc8b6f3b0380033f49061069f8cf8525f2dd6c01c12e04e9b3</cbc:UUID>
    <cbc:IssueDate>2019-04-05</cbc:IssueDate>
    <cbc:DocumentType>ApplicationResponse</cbc:DocumentType>
    <cac:Attachment>
    <cac:ExternalReference>
    <cbc:MimeCode>text/xml</cbc:MimeCode>
    <cbc:EncodingCode>UTF-8</cbc:EncodingCode>
    <cbc:Description><![CDATA[acá se coloca el ApplicationResponse en formato xml]]></cbc:Description>
    </cac:ExternalReference>
    </cac:Attachment>
    <cac:ResultOfVerification>
    <cbc:ValidatorID>Unidad Especial Dirección de Impuestos y Aduanas Nacionales</cbc:ValidatorID>
    <cbc:ValidationResultCode>02</cbc:ValidationResultCode>
    <cbc:ValidationDate>2019-04-16</cbc:ValidationDate>
    <cbc:ValidationTime>13:23:45-05:00</cbc:ValidationTime>
    </cac:ResultOfVerification>
    </cac:DocumentReference>
    </cac:ParentDocumentLineReference>
    </AttachedDocument>
    """

    nsmap = {
        "xmlns": "urn:oasis:names:specification:ubl:schema:xsd:Invoice-2",
        "xmlns:cac": "urn:oasis:names:specification:ubl:schema:xsd:CommonAggregateComponents-2",
        "xmlns:cbc": "urn:oasis:names:specification:ubl:schema:xsd:CommonBasicComponents-2",
        "xmlns:ext": "urn:oasis:names:specification:ubl:schema:xsd:CommonExtensionComponents-2",
        "xmlns:sts": "dian:gov:co:facturaelectronica:Structures-2-1",
        "xmlns:xades": "http://uri.etsi.org/01903/v1.3.2#",
        "xmlns:xades141": "http://uri.etsi.org/01903/v1.4.1#",
        "xmlns:ds": "http://www.w3.org/2000/09/xmldsig#",
        "xmlns:xsi": "http://www.w3.org/2001/XMLSchema-instance",
        "xsi:schemaLocation": "urn:oasis:names:specification:ubl:schema:xsd:Invoice-2     http://docs.oasis-open.org/ubl/os-UBL-2.1/xsd/maindoc/UBL-Invoice-2.1.xsd"
    }

    
    def Attachment(invoice, xml_attach_string=xml_attach_string, nsmap=nsmap):
        xml_soup_att = BeautifulStoneSoup(xml_attach_string)

        def unescape(s):
            s = s.replace("&lt;", "<")
            s = s.replace("&gt;", ">")
            # this has to be last:
            s.replace("&amp;", "&")
            return s        
        
        xml_file = io.BytesIO(invoice.fe_xml_file)

        try:
            with ZipFile(xml_file) as zf:
                for file in zf.namelist():
                    with zf.open(file) as f:
                        xml_file =  f.read()
        except:
            xml_file = invoice.fe_xml_file
        
        xml_soup_de = BeautifulStoneSoup(xml_file)

        if xml_soup_de.Invoice:
            xml_soup_de.Invoice.attrs.clear()
            xml_soup_de.Invoice.attrs.update(nsmap)
            uuid = xml_soup_de.Invoice.UUID.string 
            issue_date = xml_soup_de.Invoice.IssueDate.string
            issue_time = xml_soup_de.Invoice.IssueTime.string
            invoice_id = xml_soup_de.Invoice.ID.string
            party_name = xml_soup_de.Invoice.AccountingSupplierParty.Party.PartyName.Name.string
            party_company = xml_soup_de.Invoice.AccountingSupplierParty.Party.PartyTaxScheme.CompanyID.string
            party_tax = xml_soup_de.Invoice.AccountingSupplierParty.TaxLevelCode.string
            party_dv = xml_soup_de.Invoice.AccountingSupplierParty.Party.CompanyID['schemeID']
            customer_party = xml_soup_de.Invoice.AccountingCustomerParty.Party.PartyName.Name.string
            customer_id = xml_soup_de.Invoice.AccountingCustomerParty.Party.PartyTaxScheme.CompanyID.string
            customer_tax = xml_soup_de.Invoice.AccountingCustomerParty.TaxLevelCode.string
            customer_dv = xml_soup_de.Invoice.AccountingCustomerParty.Party.CompanyID['schemeID']
            InPe = xml_soup_de.find('InvoicePeriod')
        elif xml_soup_de.CreditNote:
            xml_soup_de.CreditNote.attrs.clear()
            xml_soup_de.CreditNote.attrs.update(nsmap)
            uuid = xml_soup_de.CreditNote.UUID.string 
            issue_date = xml_soup_de.CreditNote.IssueDate.string
            issue_time = xml_soup_de.CreditNote.IssueTime.string
            invoice_id = xml_soup_de.CreditNote.ID.string
            party_name = xml_soup_de.CreditNote.AccountingSupplierParty.Party.PartyName.Name.string
            party_company = xml_soup_de.CreditNote.AccountingSupplierParty.Party.PartyTaxScheme.CompanyID.string
            party_tax = xml_soup_de.CreditNote.AccountingSupplierParty.TaxLevelCode.string
            party_dv = xml_soup_de.CreditNote.AccountingSupplierParty.Party.CompanyID['schemeID']
            customer_party = xml_soup_de.CreditNote.AccountingCustomerParty.Party.PartyName.Name.string
            customer_id = xml_soup_de.CreditNote.AccountingCustomerParty.Party.PartyTaxScheme.CompanyID.string
            customer_tax = xml_soup_de.CreditNote.AccountingCustomerParty.TaxLevelCode.string
            customer_dv = xml_soup_de.CreditNote.AccountingCustomerParty.Party.CompanyID['schemeID']

            InPe = xml_soup_de.find('CreditNotePeriod')

        for d in xml_soup_de.find_all("Signature"):
            d.prefix = "ds"
            for i in d.find_all():
                i.prefix = "ds"

        UUID = uuid
        IssueDate = issue_date
        IssueTime = issue_time
        InvoiceID = invoice_id
       
        if invoice.reference:
            ORTag = xml_soup_de.new_tag('cac:OrderReference')
            InPe.insert_after(ORTag)
            OR = xml_soup_de.find('cac:OrderReference')
            ORIDTag = xml_soup_de.new_tag('cbc:ID')
            ORIDTag.string = invoice.reference
            ORIssueDateTag = xml_soup_de.new_tag('cbc:IssueDate')
            ORIssueDateTag.string = IssueDate
            OR.insert(0, ORIDTag)
            OR.insert(1, ORIssueDateTag)

        CDATA = "<![CDATA[{}]]>".format(xml_soup_de)

        xml_soup_att.ID.string =  UUID
        xml_soup_att.IssueDate.string = IssueDate
        xml_soup_att.IssueTime.string = IssueTime
        xml_soup_att.ParentDocumentID.string = InvoiceID
        xml_soup_att.SenderParty.PartyTaxScheme.RegistrationName.string = \
                party_name
        xml_soup_att.SenderParty.PartyTaxScheme.CompanyID.string = \
                party_company
        xml_soup_att.SenderParty.PartyTaxScheme.TaxLevelCode.string = \
                party_tax
        xml_soup_att.SenderParty.CompanyID['schemeID'] = party_dv
        xml_soup_att.ReceiverParty.PartyTaxScheme.RegistrationName.string = \
                customer_party 
        xml_soup_att.ReceiverParty.PartyTaxScheme.CompanyID.string = \
                customer_id
        xml_soup_att.ReceiverParty.PartyTaxScheme.TaxLevelCode.string = \
                customer_tax
        xml_soup_att.ReceiverParty.CompanyID['schemeID'] = customer_dv
        xml_soup_att.UUID.string = UUID
        xml_soup_att.ParentDocumentLineReference.DocumentReference.ID.string = \
            InvoiceID
        xml_soup_att.ParentDocumentLineReference.DocumentReference.IssueDate.string = \
            IssueDate
        xml_soup_att.ParentDocumentLineReference.ValidationDate.string = \
            IssueDate
        xml_soup_att.ParentDocumentLineReference.ValidationTime.string = \
            IssueTime
        xml_soup_att.Description.string = CDATA
        xml_soup_att.ParentDocumentLineReference.Description.string = CDATA    
        xml_final = io.StringIO(xml_soup_att.decode())
        xml_final_unescape = unescape(xml_final.read())

        return xml_final_unescape
